#include <WiFi.h>
#include <WiFiUdp.h>
#include <OSCMessage.h>
#include <OSCBundle.h>

#include "AiEsp32RotaryEncoder.h"
#include "Arduino.h"
#include "settings.h"

AiEsp32RotaryEncoder rotaryEncoder = AiEsp32RotaryEncoder(ROTARY_ENCODER_A_PIN, ROTARY_ENCODER_B_PIN, ROTARY_ENCODER_BUTTON_PIN, ROTARY_ENCODER_VCC_PIN);
int16_t encoderValue = 0;


//--------------------Includes-------------------------
#include "ArduinoTimer.h" // If missing, install the MegunoLink library:
//"Sketch"-> "Include Library" -> "Manage Libraries..." -> search and install

#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);


#if (SSD1306_LCDHEIGHT != 64)
#error("Height incorrect, please fix Adafruit_SSD1306.h!");
#endif

//--------------------Class Header-------------------------
// Delay-based simple Button Debounce Class
class DebouncedSwitch
{
    const int m_uDebounceDelayMs = 25; // Increase if bouncing still occurs
    const int m_SwitchPin;
    bool m_bRecentState;
    bool m_bLastSteadyState;
    ArduinoTimer TransitionTimer;

  public:
    DebouncedSwitch(int SwitchPin, bool UseInternalPullup);
    bool Read();
};

//--------------------Class Body-------------------------
DebouncedSwitch::DebouncedSwitch(int SwitchPin, bool UseInternalPullup)
  : m_SwitchPin(SwitchPin)
{
  if (UseInternalPullup)
  {
    pinMode(m_SwitchPin, INPUT);
  }
  else
  {
    pinMode(m_SwitchPin, INPUT);
  }

  m_bRecentState = digitalRead(m_SwitchPin);
  m_bLastSteadyState = m_bRecentState;
}

// Reads pin and returns debounced state
bool DebouncedSwitch::Read()
{
  bool CurrentState = digitalRead(m_SwitchPin);

  if (CurrentState != m_bLastSteadyState) // potential change in steady state
  {
    if (CurrentState != m_bRecentState)
    {
      TransitionTimer.Reset(); // new transition detected, wait for bounces
    }
    else if (TransitionTimer.TimePassed_Milliseconds(m_uDebounceDelayMs, false))
    {
      m_bLastSteadyState = CurrentState; // bouncing should be over by now
    }
  }

  m_bRecentState = CurrentState;

  return m_bLastSteadyState;
}

DebouncedSwitch MyButton(ButtonPin, true);


int test_limits = 4;

void rotary_onButtonClick() {
  //rotaryEncoder.reset();
  //rotaryEncoder.disable();
  rotaryEncoder.setBoundaries(-test_limits, test_limits, false);
  test_limits *= 2;
}

void rotary_loop() {
  //first lets handle rotary encoder button click
  if (rotaryEncoder.currentButtonState() == BUT_RELEASED) {
    //we can process it here or call separate function like:
    rotary_onButtonClick();
  }

  //lets see if anything changed
  int16_t encoderDelta = rotaryEncoder.encoderChanged();

  //optionally we can ignore whenever there is no change
  if (encoderDelta == 0) return;

  //for some cases we only want to know if value is increased or decreased (typically for menu items)
  if (encoderDelta > 0) Serial.print("+");
  if (encoderDelta < 0) Serial.print("-");

  //for other cases we want to know what is current value. Additionally often we only want if something changed
  //example: when using rotary encoder to set termostat temperature, or sound volume etc

  //if value is changed compared to our last read
  if (encoderDelta != 0) {
    refreshDisplay = true;
    //now we need current value
    encoderValue = rotaryEncoder.readEncoder();
    //process new value. Here is simple output.
    Serial.print("Value: ");
    Serial.println(encoderValue);
  }

}

//--------------------Setup-------------------------
void setup()
{
  // Setup serial for outputting button press count
  Serial.begin(115200);
  Serial.println("SIOCHAIR");

  // SSD1306
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);  // initialize with the I2C addr 0x3D (for the 128x64)
  // internally, this will display the splashscreen.
  display.display();
  display.invertDisplay(false);
  display.setRotation(2);
  //delay(2000);
  display.clearDisplay();


  // OSC
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.begin(ssid, pass);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");

  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());

  Serial.println("Starting UDP");
  Udp.begin(localPort);
  Serial.print("Local port: ");
  Serial.println(localPort);

  // BUTTON
  // Setup last state variable used to detect button changes
  LastState = MyButton.Read();

  // VIB MOTOR
  // Setup onboard LED pin for indicating button state
  pinMode(LedPin, OUTPUT);


  // ENCODER
  //we must initialize rorary encoder
  rotaryEncoder.begin();
  rotaryEncoder.setup([] {rotaryEncoder.readEncoder_ISR();});
  //optionally we can set boundaries and if values should cycle or not
  rotaryEncoder.setBoundaries(0, 1, true); //minValue, maxValue, cycle values (when max go to min and vice versa)
}


//--------------------Loop-------------------------
void loop()
{
  rotary_loop();
  if (millis() > encoderWait) rotaryEncoder.enable ();

  // Read the debounced button state:
  // using        MyButton.Read()
  // instead of   digitalRead(ButtonPin)

  bool CurrentState = MyButton.Read();

  // Turn on LED if the button is being pressed
  if (CurrentState == LOW) // level-sensitive
  {
    digitalWrite(LedPin, HIGH);
    pomoAction = true;

  } else {
    digitalWrite(LedPin, LOW);
  }

  // Increment and print the count at the start of each button press
  if (CurrentState == LOW && CurrentState != LastState) // edge-sensitive
  {
    //pomoAction = true;
    //refreshDisplay = true;

    Count++;
    Serial.print("Count: ");
    Serial.println(Count);

  }

  // Store button state so changes can be detected
  LastState = CurrentState;

  if (pomoCount >= pomoCountMax) {
    Serial.println("reached max. reset pomoCount");
    pomoAction = false;
    pomoReset = true;
    display.clearDisplay();
    display.setTextSize(2);
    display.setTextColor(WHITE);
    display.setCursor(10, 5);
    display.println("take a break");  // Clear the buffer.
    display.display();
    digitalWrite(LedPin, HIGH);
    delay(200);
    digitalWrite(LedPin, LOW);
    delay(500);
    digitalWrite(LedPin, HIGH);
    delay(200);
    digitalWrite(LedPin, LOW);
    delay(500);
    refreshDisplay = true;
    encoderValue = 2;



  }


  if (encoderValue == 1) {
    //pomoAction = true;
    //pomoCount = 0;
    //Serial.println("count2");
  }
  else if (encoderValue != 2) {
    selectAction == 3;
    pomoAction = false;
    pomoReset = true;
  }
  if (Count >= maxCount) {
    Count = 0;
  }
  //pomoCurrentTime = millis();
  if (pomoReset == true) {
    pomoReset = false;
    pomoCount = 0;
  }



  if ((millis() - pomoCurrentTime) > 1000 && pomoAction == true && encoderValue == 1) {
    //Serial.println(pomoCurrentTime);

    if (pomoCount < pomoCountMax) {
      pomoCount ++;
      Serial.printf("pomoCount: %d \n", pomoCount);
      refreshDisplay = true;
    }

    pomoCurrentTime = millis();
  }
  if (refreshDisplay == true) {
    uint32_t seconds = pomoCount;
    display.clearDisplay();
    display.setTextSize(2);
    display.setTextColor(WHITE);
    display.setCursor(10, 5);

    if (encoderValue != 1) {
      display.println("SIOCHAIR");  // Clear the buffer.
      refreshDisplay = false;
    }
    else if (encoderValue == 1) {
      uint32_t s = seconds % 60;
      seconds = (seconds - s) / 60;
      uint32_t m = seconds % 60;

      display.println("POMODORE");  // Clear the buffer.
      display.setTextSize(3);
      display.setTextColor(WHITE);
      display.setCursor(10, 30);
      display.println(String(m) + ":" + String(s));  // Clear the buffer.
      //display.setTextSize(2);
      //display.println("secs");  // Clear the buffer.

      refreshDisplay = false;

    }
    else {
      display.println(encoderValue);  // Clear the buffer.
      refreshDisplay = false;

    }
    display.display();
    delay(100);
  }
  if ((millis() - oscSendTime) > oscInterval) {
    uint8_t t0 = touchRead(T0);
    uint8_t t2 = touchRead(T2);
    uint8_t t3 = touchRead(T3);
    uint8_t t4 = touchRead(T4);
    uint8_t t6 = touchRead(T6);
    uint8_t t7 = touchRead(T7);
    uint8_t t8 = touchRead(T8);
    uint8_t t9 = touchRead(T9);
    //declare the bundle
    OSCBundle bndl;
    //BOSCBundle's add' returns the OSCMessage so the message's 'add' can be composed together
    bndl.add("/touch/0").add((int32_t)t0);
    bndl.add("/touch/2").add((int32_t)t2);
    bndl.add("/touch/3").add((int32_t)t3);
    bndl.add("/touch/4").add((int32_t)t4);
    bndl.add("/touch/6").add((int32_t)t6);
    bndl.add("/touch/7").add((int32_t)t7);
    bndl.add("/touch/8").add((int32_t)t8);
    bndl.add("/touch/9").add((int32_t)t9);

    Udp.beginPacket(outIp, outPort);
    bndl.send(Udp);
    Udp.endPacket();
    bndl.empty();

    oscSendTime = millis();
  }
}
